@@ -23,6 +23,8 @@ getParametersForFileType <- function (fileType = NA, format = NA, singleFile = N
     return (parameters)
 }
 
+#' @rdname files
+#' @export
 identifyImageFileNames <- function (fileName, fileType = NULL, errorIfMissing = TRUE)
 {
     suffixes <- union(.FileTypes$headerSuffixes, .FileTypes$imageSuffixes)
@@ -34,15 +36,27 @@ identifyImageFileNames <- function (fileName, fileType = NULL, errorIfMissing =
     
     if (length(headersExist) < 1 || length(imagesExist) < 1)
     {
+        for (i in seq_along(.Workspace$pathHandlers))
+        {
+            if (fileName %~% names(.Workspace$pathHandlers)[i])
+            {
+                fileName <- .Workspace$pathHandlers[[i]](fileName)
+                if (is.null(fileName))
+                    report(OL$Error, "Custom path handler could not resolve file name: #{fileName}")
+                else
+                    return(identifyImageFileNames(fileName, fileType=fileType, errorIfMissing=errorIfMissing))
+            }
+        }
+        
         if (errorIfMissing)
-            report(OL$Error, "Complete image file does not exist: ", fileName)
+            report(OL$Error, "Complete image file does not exist: #{fileName}")
         else
             return (NULL)
     }
     if (length(headersExist) > 1 || length(imagesExist) > 1)
     {
         if (errorIfMissing)
-            report(OL$Error, "Multiple compatible image files exist: ", fileName)
+            report(OL$Error, "Multiple compatible image files exist: #{fileName}")
         else
             return (NULL)
     }
@@ -66,6 +80,8 @@ identifyImageFileNames <- function (fileName, fileType = NULL, errorIfMissing =
     return (fileNames)
 }
 
+#' @rdname files
+#' @export
 imageFileExists <- function (fileName, fileType = NULL)
 {
     return (sapply(fileName, function (file) {
@@ -73,7 +89,9 @@ imageFileExists <- function (fileName, fileType = NULL)
     }))
 }
 
-removeImageFilesWithName <- function (fileName)
+#' @rdname files
+#' @export
+removeImageFiles <- function (fileName)
 {
     fileName <- expandFileName(fileName)
     suffixes <- union(.FileTypes$headerSuffixes, .FileTypes$imageSuffixes)    
@@ -81,6 +99,8 @@ removeImageFilesWithName <- function (fileName)
     unlink(files)
 }
 
+#' @rdname files
+#' @export
 symlinkImageFiles <- function (from, to, overwrite = FALSE, relative = TRUE)
 {
     if (length(from) != length(to))
@@ -94,7 +114,8 @@ symlinkImageFiles <- function (from, to, overwrite = FALSE, relative = TRUE)
         currentSource <- unique(c(info$headerFile, info$imageFile))
         currentTarget <- unique(ensureFileSuffix(expandFileName(to[i]), c(info$headerSuffix,info$imageSuffix), strip=suffixes))
         
-        if (overwrite && any(file.exists(currentTarget)))
+        # NB: file.exists() requires the target of an existing link to exist
+        if (overwrite && any(!is.na(Sys.readlink(currentTarget))))
             unlink(currentTarget)
         if (relative)
         {
@@ -106,6 +127,8 @@ symlinkImageFiles <- function (from, to, overwrite = FALSE, relative = TRUE)
     }
 }
 
+#' @rdname files
+#' @export
 copyImageFiles <- function (from, to, overwrite = FALSE, deleteOriginals = FALSE)
 {
     if (length(from) != length(to))
@@ -126,7 +149,7 @@ copyImageFiles <- function (from, to, overwrite = FALSE, deleteOriginals = FALSE
         success <- file.copy(unique(currentSource), unique(currentTarget), overwrite=overwrite)
         
         if (all(success) && deleteOriginals)
-            removeImageFilesWithName(from[i])
+            removeImageFiles(from[i])
     }
 }
 
@@ -191,6 +214,107 @@ chooseDataTypeForImage <- function (image, format)
     return (list(code=code, type=rType, size=size, isSigned=isSigned))
 }
 
+#' Working with MRI images stored in NIfTI, Analyze and MGH formats
+#' 
+#' Functions for reading, writing, locating, copying and removing MRI images
+#' stored in NIfTI, Analyze and MGH formats.
+#' 
+#' NIfTI and Analyze are related formats for storing magnetic resonance images.
+#' NIfTI is a more recent extension of Analyze, and contains more specific
+#' information about, for example, the orientation of the image. Its use is
+#' therefore recommended where possible. MGH format is used by the popular
+#' image processing package FreeSurfer. These formats use a number of different
+#' file extensions, but the details are abstracted away from the user by these
+#' functions.
+#' 
+#' TractoR does not allow for files with the same basic name using multiple
+#' Analyze/NIfTI/MGH formats in a single directory (e.g. \code{"foo.nii"} AND
+#' \code{"foo.img"}), and these functions will produce an error if multiple
+#' compatible files exist.
+#' 
+#' Suitable values for \code{fileType} (and the \code{tractorFileType} option,
+#' which is used as a default) are \code{ANALYZE}, \code{NIFTI},
+#' \code{NIFTI_PAIR} (the two-file NIfTI format), \code{MGH},
+#' \code{ANALYZE_GZ}, \code{NIFTI_GZ}, \code{NIFTI_PAIR_GZ} and \code{MGH_GZ}.
+#' The latter four are gzipped versions of the former four. \code{NIFTI_GZ} is
+#' recommended unless there is a need for one of the others. This is the
+#' default value for the \code{tractorFileType} option, but that can be changed
+#' using a call to \code{\link{options}}, or by setting the
+#' \code{TRACTOR_FILETYPE} environment variable before loading the tractor.base
+#' package.
+#' 
+#' Since multiple files may be involved, copying, moving or symlinking images
+#' is not trivial. \code{copyImageFiles} and \code{symlinkImageFiles} are
+#' wrappers around the standard functions \code{\link{file.copy}} and
+#' \code{\link{file.symlink}} which handle this complexity.
+#' 
+#' \code{registerPathHandler} allows special syntaxes to be used for image
+#' paths, and is for advanced use only.
+#' 
+#' @param fileName,from,to File names, with or without appropriate extension.
+#' @param image An \code{\linkS4class{MriImage}} object.
+#' @param fileType A character vector of length one, giving the file type
+#'   required or expected. If this option is missing, the file type used for
+#'   writing images will be taken from the \code{tractorFileType} option. See
+#'   Details.
+#' @param metadataOnly Logical value: if \code{TRUE}, only metadata are read
+#'   into the object.
+#' @param volumes An optional integer vector specifying a subset of volumes to
+#'   read (generally to save memory). If given, only the requested volumes in
+#'   the 4D file will be read.
+#' @param sparse Logical value: should the image data be stored in a
+#'  \code{\linkS4class{SparseArray}} object?
+#' @param mask An optional \code{\linkS4class{MriImage}} object representing a
+#'   mask, outside of which the image to be read should be considered to be
+#'   zero. This can be used to save memory when only a small part of a large
+#'   image is of interest. Ignored if \code{sparse} is not \code{TRUE}.
+#' @param reorder Logical value: should the image data be reordered to LAS?
+#'   This is recommended in most circumstances.
+#' @param overwrite Logical value: overwrite an existing image file? For
+#'   \code{writeImageFile}, an error will be raised if there is an existing
+#'   file and this is set to FALSE.
+#' @param maxSize If not \code{NULL}, the maximum number of bytes per pixel to
+#'   use when storing the data. This can lead to a substantial loss of
+#'   precision, and is usually not desirable. Only used when writing to the
+#'   NIfTI file format.
+#' @param errorIfMissing Logical value: raise an error if no suitable files
+#'   were found?
+#' @param deleteOriginals Logical value: if \code{TRUE}, \code{copyImageFiles}
+#'   performs a move rather than a copy.
+#' @param relative Logical value: if \code{TRUE}, the path stored in the
+#'   symlink will be relative (e.g. \code{"../some_dir/some_image.nii"}) rather
+#'   than absolute (e.g. \code{"/path/to/some_dir/some_image.nii"}).
+#' @param regex A regular expression.
+#' @param handler A function taking and returning a string.
+#' @return \code{readImageFile} returns an \code{\linkS4class{MriImage}}
+#'   object. \code{imageFileExists} returns \code{TRUE} if an existing file
+#'   with the specified name exists (all file extensions are checked), and
+#'   \code{FALSE} otherwise. \code{removeImageFiles} returns the result of
+#'   \code{\link{unlink}} applied to all relevant files. \code{writeImageFile}
+#'   and \code{identifyImageFileNames} return a list with the following elements,
+#'   describing the identified or written files:
+#'   \describe{
+#'     \item{fileStem}{The file name without extension.}
+#'     \item{headerFile}{The full header file name.}
+#'     \item{imageFile}{The full image file name.}
+#'     \item{format}{The format of the files (\code{"Nifti"}, \code{"Analyze"}
+#'       or \code{"Mgh"}). Not returned by \code{writeImageFile}.}
+#'   }
+#'   \code{copyImageFiles} and \code{symlinkImageFiles} are called for their
+#'   side effects.
+#' 
+#' @author Jon Clayden
+#' @seealso The NIfTI-1 standard (\url{http://nifti.nimh.nih.gov/nifti-1}) and
+#'   \code{\linkS4class{MriImage}}.
+#' @references Please cite the following reference when using TractoR in your
+#' work:
+#' 
+#' J.D. Clayden, S. Muñoz Maniega, A.J. Storkey, M.D. King, M.E. Bastin & C.A.
+#' Clark (2011). TractoR: Magnetic resonance imaging and tractography with R.
+#' Journal of Statistical Software 44(8):1-18.
+#' \url{http://www.jstatsoft.org/v44/i08/}.
+#' @rdname files
+#' @export
 readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volumes = NULL, sparse = FALSE, mask = NULL, reorder = TRUE)
 {
     fileNames <- identifyImageFileNames(fileName, fileType)
@@ -332,26 +456,21 @@ readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volu
     report(OL$Debug, "Image orientation is ", xformToOrientation(info$storageMetadata$xformMatrix))
     
     # The origin is world position (0,0,0); the xform is a voxel-to-world affine matrix
-    origin <- c(rep(1,min(nDims,3)), rep(0,max(0,nDims-3)))
+    origin <- rep(1,3)
     if (equivalent(dim(info$storageMetadata$xformMatrix), c(4,4)))
     {
         tempOrigin <- (solve(info$storageMetadata$xformMatrix) %*% c(0,0,0,1)) + 1
-        origin[1:min(3,nDims)] <- tempOrigin[1:min(3,nDims)]
+        origin <- tempOrigin[1:3]
     }
     
     image <- MriImage$new(imageDims=dims, voxelDims=voxelDims, voxelDimUnits=info$imageMetadata$voxelUnit, source=info$imageMetadata$source, origin=origin, storedXform=info$storageMetadata$xformMatrix, reordered=FALSE, tags=info$imageMetadata$tags, data=data)
     
     if (reorder)
-        image <- newMriImageByReordering(image)
+        image <- reorderMriImage(image)
     
     invisible (image)
 }
 
-newMriImageFromFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volumes = NULL, sparse = FALSE, mask = NULL, reorder = TRUE)
-{
-    readImageFile(fileName, fileType, metadataOnly, volumes, sparse, mask, reorder)
-}
-
 writeImageData <- function (image, connection, type, size, endian = .Platform$endian)
 {
     if (!is(image, "MriImage"))
@@ -382,7 +501,9 @@ writeImageData <- function (image, connection, type, size, endian = .Platform$en
     }
 }
 
-writeImageFile <- function (image, fileName = NULL, fileType = NA, overwrite = TRUE)
+#' @rdname files
+#' @export
+writeImageFile <- function (image, fileName = NULL, fileType = NA, overwrite = TRUE, maxSize = NULL)
 {
     if (!is(image, "MriImage"))
         report(OL$Error, "The specified image is not an MriImage object")
@@ -417,16 +538,24 @@ writeImageFile <- function (image, fileName = NULL, fileType = NA, overwrite = T
         report(OL$Error, "An unreordered image can only be written to NIfTI format")
     
     if (params$format == "Analyze")
-        writeMriImageToAnalyze(image, fileNames, gzipped=params$gzipped)
+        writeAnalyze(image, fileNames, gzipped=params$gzipped)
     else if (params$format == "Nifti")
-        writeMriImageToNifti(image, fileNames, gzipped=params$gzipped)
+        writeNifti(image, fileNames, gzipped=params$gzipped, maxSize=maxSize)
     else if (params$format == "Mgh")
-        writeMriImageToMgh(image, fileNames, gzipped=params$gzipped)
+        writeMgh(image, fileNames, gzipped=params$gzipped)
     
     invisible (fileNames)
 }
 
-writeMriImageToFile <- function (image, fileName = NULL, fileType = NA, overwrite = TRUE)
+#' @rdname files
+#' @export
+registerPathHandler <- function (regex, handler)
 {
-    writeImageFile(image, fileName, fileType, overwrite)
+    if (!is.character(regex) || length(regex) != 1)
+        report(OL$Error, "Regular expression should be specified as a character string")
+    
+    handler <- match.fun(handler)
+    .Workspace$pathHandlers[[regex]] <- handler
+    
+    invisible(NULL)
 }
