@@ -1,3 +1,9 @@
+#' @import methods ore reportr
+#' @importFrom grDevices col2rgb colorRamp dev.cur dev.off gray heat.colors rainbow
+#' @importFrom graphics image layout lines locator par plot strwidth text
+#' @importFrom stats na.omit
+NULL
+
 .onLoad <- function (libname, pkgname)
 {
     if (is.null(getOption("tractorFileType")))
